@@ -27,8 +27,8 @@ import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.ScheduledExecutorService;
 
-import io.airlift.airline.Command;
-import io.airlift.airline.Option;
+import com.github.rvesse.airline.annotations.Command;
+import com.github.rvesse.airline.annotations.Option;
 import org.apache.activemq.artemis.api.core.ActiveMQBuffer;
 import org.apache.activemq.artemis.api.core.ActiveMQBuffers;
 import org.apache.activemq.artemis.api.core.SimpleString;
@@ -64,24 +64,24 @@ import org.apache.activemq.artemis.utils.actors.ArtemisExecutor;
 import org.apache.activemq.artemis.utils.collections.LinkedList;
 import org.apache.activemq.artemis.utils.collections.LinkedListIterator;
 
-@Command(name = "print", description = "Print data records information (WARNING: don't use while a production server is running)")
+@Command(name = "print", description = "Print data records information. WARNING: don't use while a production server is running.")
 public class PrintData extends DBOption {
 
 
-   @Option(name = "--safe", description = "It will print your data structure without showing your data")
+   @Option(name = "--safe", description = "Print your data structure without showing your data.")
    private boolean safe = false;
 
 
-   @Option(name = "--reclaimed", description = "This option will try to print as many records as possible from reclaimed files")
+   @Option(name = "--reclaimed", description = "Try to print as many records as possible from reclaimed files.")
    private boolean reclaimed = false;
 
-   @Option(name = "--max-pages", description = "It will print your data structure without showing your data")
+   @Option(name = "--max-pages", description = "Maximum number of pages to read. Default: unlimited (-1).")
    private int maxPages = -1;
 
-   @Option(name = "--skip-bindings", description = "It will ignore printing the bindings journal")
+   @Option(name = "--skip-bindings", description = "Do not print data from the bindings journal.")
    private boolean skipBindings = false;
 
-   @Option(name = "--skip-journal", description = "It will ignore printing the messages journal")
+   @Option(name = "--skip-journal", description = "Do not print data from the messages journal.")
    private boolean skipJournal = false;
 
    private static final String BINDINGS_BANNER = "B I N D I N G S  J O U R N A L";
@@ -238,7 +238,7 @@ public class PrintData extends DBOption {
       try {
 
          final StorageManager sm = new NullStorageManager();
-         PagingStoreFactory pageStoreFactory = new PagingStoreFactoryNIO(sm, pageDirectory, 1000L, scheduled, execfactory, false, null);
+         PagingStoreFactory pageStoreFactory = new PagingStoreFactoryNIO(sm, pageDirectory, 1000L, scheduled, execfactory, execfactory, false, null);
          HierarchicalRepository<AddressSettings> addressSettingsRepository = new HierarchicalObjectRepository<>();
          addressSettingsRepository.setDefault(new AddressSettings());
          PagingManager manager = new PagingManagerImpl(pageStoreFactory, addressSettingsRepository);
@@ -279,7 +279,7 @@ public class PrintData extends DBOption {
             folder = pgStore.getFolder();
             out.println("####################################################################################################");
             out.println("Exploring store " + store + " folder = " + folder);
-            int pgid = (int) pgStore.getFirstPage();
+            long pgid = pgStore.getFirstPage();
 
             out.println("Number of pages ::" + pgStore.getNumberOfPages() + ", Current writing page ::" + pgStore.getCurrentWritingPage());
             for (int pg = 0; pg < pgStore.getNumberOfPages(); pg++) {
