@@ -22,7 +22,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.Executor;
-import java.util.concurrent.TimeUnit;
 
 import org.apache.activemq.artemis.api.core.ActiveMQBuffer;
 import org.apache.activemq.artemis.api.core.ActiveMQException;
@@ -40,6 +39,8 @@ import org.apache.activemq.artemis.core.paging.PagingManager;
 import org.apache.activemq.artemis.core.paging.PagingStore;
 import org.apache.activemq.artemis.core.paging.cursor.PagePosition;
 import org.apache.activemq.artemis.core.persistence.config.PersistedAddressSetting;
+import org.apache.activemq.artemis.core.persistence.config.PersistedBridgeConfiguration;
+import org.apache.activemq.artemis.core.persistence.config.PersistedConnector;
 import org.apache.activemq.artemis.core.persistence.config.PersistedDivertConfiguration;
 import org.apache.activemq.artemis.core.persistence.config.PersistedKeyValuePair;
 import org.apache.activemq.artemis.core.persistence.config.PersistedRole;
@@ -79,6 +80,9 @@ public interface StorageManager extends IDGenerator, ActiveMQComponent {
       return Long.MAX_VALUE;
    }
 
+   default void recoverLargeMessagesOnFolder(Set<Long> files) throws Exception {
+   }
+
    default SequentialFileFactory getJournalSequentialFileFactory() {
       return null;
    }
@@ -114,11 +118,11 @@ public interface StorageManager extends IDGenerator, ActiveMQComponent {
 
    // Message related operations
 
-   void pageClosed(SimpleString storeName, int pageNumber);
+   void pageClosed(SimpleString storeName, long pageNumber);
 
-   void pageDeleted(SimpleString storeName, int pageNumber);
+   void pageDeleted(SimpleString storeName, long pageNumber);
 
-   void pageWrite(PagedMessage message, int pageNumber);
+   void pageWrite(PagedMessage message, long pageNumber);
 
    void afterCompleteOperations(IOCallback run);
 
@@ -145,30 +149,6 @@ public interface StorageManager extends IDGenerator, ActiveMQComponent {
     */
    void waitOnOperations() throws Exception;
 
-   /**
-    * We need a safeguard in place to avoid too much concurrent IO happening on Paging, otherwise
-    * the system may become unresponsive if too many destinations are reading all the same time.
-    * This is called before we read, so we can limit concurrent reads
-    *
-    * @throws Exception
-    */
-   void beforePageRead() throws Exception;
-
-   /**
-    * Like {@link #beforePageRead()} but return {@code true} if acquired within {@code timeout},
-    * {@code false} otherwise.
-    */
-   boolean beforePageRead(long timeout, TimeUnit unit) throws InterruptedException;
-
-   /**
-    * We need a safeguard in place to avoid too much concurrent IO happening on Paging, otherwise
-    * the system may become unresponsive if too many destinations are reading all the same time.
-    * This is called after we read, so we can limit concurrent reads
-    *
-    * @throws Exception
-    */
-   void afterPageRead() throws Exception;
-
    /**
     * AIO has an optimized buffer which has a method to release it
     * instead of the way NIO will release data based on GC.
@@ -305,12 +285,24 @@ public interface StorageManager extends IDGenerator, ActiveMQComponent {
 
    void deletePageTransactional(long recordID) throws Exception;
 
+   default JournalLoadInformation loadMessageJournal(PostOffice postOffice,
+                                             PagingManager pagingManager,
+                                             ResourceManager resourceManager,
+                                             Map<Long, QueueBindingInfo> queueInfos,
+                                             Map<SimpleString, List<Pair<byte[], Long>>> duplicateIDMap,
+                                             Set<Pair<Long, Long>> pendingLargeMessages,
+                                             List<PageCountPending> pendingNonTXPageCounter,
+                                             JournalLoader journalLoader) throws Exception {
+      return loadMessageJournal(postOffice, pagingManager, resourceManager, queueInfos, duplicateIDMap, pendingLargeMessages, null, pendingNonTXPageCounter, journalLoader);
+   }
+
    JournalLoadInformation loadMessageJournal(PostOffice postOffice,
                                              PagingManager pagingManager,
                                              ResourceManager resourceManager,
                                              Map<Long, QueueBindingInfo> queueInfos,
                                              Map<SimpleString, List<Pair<byte[], Long>>> duplicateIDMap,
                                              Set<Pair<Long, Long>> pendingLargeMessages,
+                                             Set<Long> largeMessagesInFolder,
                                              List<PageCountPending> pendingNonTXPageCounter,
                                              JournalLoader journalLoader) throws Exception;
 
@@ -372,6 +364,18 @@ public interface StorageManager extends IDGenerator, ActiveMQComponent {
 
    List<PersistedDivertConfiguration> recoverDivertConfigurations();
 
+   void storeBridgeConfiguration(PersistedBridgeConfiguration persistedBridgeConfiguration) throws Exception;
+
+   void deleteBridgeConfiguration(String bridgeName) throws Exception;
+
+   List<PersistedBridgeConfiguration> recoverBridgeConfigurations();
+
+   void storeConnector(PersistedConnector persistedConnector) throws Exception;
+
+   void deleteConnector(String connectorName) throws Exception;
+
+   List<PersistedConnector> recoverConnectors();
+
    void storeUser(PersistedUser persistedUser) throws Exception;
 
    void deleteUser(String username) throws Exception;
@@ -436,11 +440,6 @@ public interface StorageManager extends IDGenerator, ActiveMQComponent {
 
    /**
     * Write message to page if we are paging.
-    * <p>
-    * This is primarily a {@link PagingStore} call, but as with any other call writing persistent
-    * data, it must go through here. Both for the sake of replication, and also to ensure that it
-    * takes the locks (storage manager and pagingStore) in the right order. Avoiding thus the
-    * creation of dead-locks.
     *
     * @return {@code true} if we are paging and have handled the data, {@code false} if the data
     * needs to be sent to the journal
