@@ -96,6 +96,20 @@ public class FileConfigurationParserTest extends ActiveMQTestBase {
       assertEquals(0, server.locateQueue(SimpleString.toSimpleString("q")).getMaxConsumers());
    }
 
+   @Test
+   public void testAddressWithNoRoutingType() throws Exception {
+      String filename = "FileConfigurationParser-addressWithNoRoutingType.xml";
+      FileConfiguration fc = new FileConfiguration();
+      FileDeploymentManager deploymentManager = new FileDeploymentManager(filename);
+      deploymentManager.addDeployable(fc);
+      try {
+         deploymentManager.readConfiguration();
+         fail();
+      } catch (IllegalArgumentException e) {
+         // expected exception when address has no routing type configured
+      }
+   }
+
    @Test
    public void testDuplicateAddressSettings() throws Exception {
       FileConfigurationParser parser = new FileConfigurationParser();
@@ -121,6 +135,18 @@ public class FileConfigurationParserTest extends ActiveMQTestBase {
       Assert.assertEquals(333, config.getClusterConfigurations().get(0).getRetryInterval());
    }
 
+   @Test
+   public void testParsingZeroIDCacheSize() throws Exception {
+      FileConfigurationParser parser = new FileConfigurationParser();
+
+      String configStr = firstPart + "<id-cache-size>0</id-cache-size>" + lastPart;
+      ByteArrayInputStream input = new ByteArrayInputStream(configStr.getBytes(StandardCharsets.UTF_8));
+
+      Configuration config = parser.parseMainConfig(input);
+
+      Assert.assertEquals(0, config.getIDCacheSize());
+   }
+
    @Test
    public void testWildcardConfiguration() throws Exception {
       FileConfigurationParser parser = new FileConfigurationParser();
@@ -381,6 +407,38 @@ public class FileConfigurationParserTest extends ActiveMQTestBase {
       AddressSettings settings = configuration.getAddressSettings().get("foo");
       Assert.assertEquals(1024, settings.getMaxReadPageBytes());
       Assert.assertEquals(33, settings.getMaxReadPageMessages());
+      Assert.assertNull(settings.getPageLimitBytes());
+      Assert.assertNull(settings.getPageLimitMessages());
+      Assert.assertNull(settings.getPageFullMessagePolicy());
+   }
+
+   @Test
+   public void testParsePageLimitSettings() throws Exception {
+      String configStr = "<configuration><address-settings>" + "\n" + "<address-setting match=\"foo\">" + "\n" + "<max-read-page-bytes>1k</max-read-page-bytes><page-limit-bytes>10G</page-limit-bytes><page-limit-messages>3221225472</page-limit-messages><page-full-policy>FAIL</page-full-policy><max-read-page-messages>33</max-read-page-messages>.\n" + "</address-setting>" + "\n" + "</address-settings></configuration>" + "\n";
+
+      FileConfigurationParser parser = new FileConfigurationParser();
+      ByteArrayInputStream input = new ByteArrayInputStream(configStr.getBytes(StandardCharsets.UTF_8));
+
+      Configuration configuration = parser.parseMainConfig(input);
+      AddressSettings settings = configuration.getAddressSettings().get("foo");
+      Assert.assertEquals(1024, settings.getMaxReadPageBytes());
+      Assert.assertEquals(33, settings.getMaxReadPageMessages());
+      Assert.assertEquals(10L * 1024 * 1024 * 1024, settings.getPageLimitBytes().longValue());
+      Assert.assertEquals(3L * 1024 * 1024 * 1024, settings.getPageLimitMessages().longValue());
+      Assert.assertEquals("FAIL", settings.getPageFullMessagePolicy().toString());
+   }
+
+   @Test
+   public void testParseMaxReadAddressSettingsAllNegative() throws Exception {
+      String configStr = "<configuration><address-settings>" + "\n" + "<address-setting match=\"foo\">" + "\n" + "<max-read-page-bytes>-1</max-read-page-bytes><max-read-page-messages>-1</max-read-page-messages>.\n" + "</address-setting>" + "\n" + "</address-settings></configuration>" + "\n";
+
+      FileConfigurationParser parser = new FileConfigurationParser();
+      ByteArrayInputStream input = new ByteArrayInputStream(configStr.getBytes(StandardCharsets.UTF_8));
+
+      Configuration configuration = parser.parseMainConfig(input);
+      AddressSettings settings = configuration.getAddressSettings().get("foo");
+      Assert.assertEquals(-1, settings.getMaxReadPageBytes());
+      Assert.assertEquals(-1, settings.getMaxReadPageMessages());
    }
 
    // you should not use K, M notations on address settings max-size-messages
@@ -477,6 +535,22 @@ public class FileConfigurationParserTest extends ActiveMQTestBase {
       Assert.assertEquals(1000, configuration.getGlobalMaxMessages());
    }
 
+   @Test
+   public void testConfigurationPersistRedelivery() throws Exception {
+      StringPrintStream stringPrintStream = new StringPrintStream();
+      PrintStream stream = stringPrintStream.newStream();
+
+      stream.println("<configuration><core>");
+      stream.println("<max-redelivery-records>0</max-redelivery-records>");
+      stream.println("</core></configuration>");
+
+      ByteArrayInputStream inputStream = new ByteArrayInputStream(stringPrintStream.getBytes());
+      FileConfigurationParser parser = new FileConfigurationParser();
+      Configuration configuration = parser.parseMainConfig(inputStream);
+
+      Assert.assertEquals(0, configuration.getMaxRedeliveryRecords());
+   }
+
    @Test
    public void testExceptionMaxSize() throws Exception {
       StringPrintStream stringPrintStream = new StringPrintStream();
@@ -500,6 +574,29 @@ public class FileConfigurationParserTest extends ActiveMQTestBase {
 
    }
 
+   @Test
+   public void testNotations() throws Exception {
+      StringPrintStream stringPrintStream = new StringPrintStream();
+      PrintStream stream = stringPrintStream.newStream();
+
+      stream.println("<configuration><core>");
+      stream.println("<global-max-size>100MiB</global-max-size>");
+      stream.println("<journal-file-size>10M</journal-file-size>");
+      stream.println("<journal-buffer-size>5Mb</journal-buffer-size>");
+      stream.println("</core></configuration>");
+
+      ByteArrayInputStream inputStream = new ByteArrayInputStream(stringPrintStream.getBytes());
+      FileConfigurationParser parser = new FileConfigurationParser();
+      Configuration configuration = parser.parseMainConfig(inputStream);
+
+      // check that suffixes were interpreted well
+      Assert.assertEquals(100 * 1024 * 1024, configuration.getGlobalMaxSize());
+      Assert.assertEquals(10 * 1024 * 1024, configuration.getJournalFileSize());
+      // one of the two will get the value
+      Assert.assertTrue(5 * 1024 * 1024 == configuration.getJournalBufferSize_AIO() ||
+                                 5 * 1024 * 1024 == configuration.getJournalBufferSize_NIO());
+   }
+
    @Test
    public void testRetentionJournalOptionsDays() throws Exception {
       testStreamDatesOption("DAYS", TimeUnit.DAYS);
