@@ -1,12 +1,12 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements. See the NOTICE file distributed with
+ * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
+ * the License.  You may obtain a copy of the License at
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -14,12 +14,12 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.activemq.artemis.tests.integration.mqtt5;
 
 import javax.jms.ConnectionFactory;
 import java.io.File;
 import java.io.IOException;
+import java.lang.invoke.MethodHandles;
 import java.security.ProtectionDomain;
 import java.util.Arrays;
 import java.util.Collection;
@@ -75,7 +75,6 @@ import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-import java.lang.invoke.MethodHandles;
 
 import static java.util.Collections.singletonList;
 import static org.apache.activemq.artemis.core.protocol.mqtt.MQTTProtocolManagerFactory.MQTT_PROTOCOL_NAME;
@@ -107,6 +106,10 @@ public class MQTT5TestSupport extends ActiveMQTestBase {
       return new MqttClient(protocol + "://localhost:" + (isUseSsl() ? getSslPort() : getPort()), clientId, new MemoryPersistence());
    }
 
+   protected org.eclipse.paho.client.mqttv3.MqttClient createPaho3_1_1Client(String clientId) throws org.eclipse.paho.client.mqttv3.MqttException {
+      return new org.eclipse.paho.client.mqttv3.MqttClient(protocol + "://localhost:" + (isUseSsl() ? getSslPort() : getPort()), clientId, new org.eclipse.paho.client.mqttv3.persist.MemoryPersistence());
+   }
+
    protected MqttAsyncClient createAsyncPahoClient(String clientId) throws MqttException {
       return new MqttAsyncClient(protocol + "://localhost:" + (isUseSsl() ? getSslPort() : getPort()), clientId, new MemoryPersistence());
    }
@@ -131,6 +134,9 @@ public class MQTT5TestSupport extends ActiveMQTestBase {
    protected String noprivUser = "noprivs";
    protected String noprivPass = "noprivs";
 
+   protected String createAddressUser = "createAddress";
+   protected String createAddressPass = "createAddress";
+
    protected String browseUser = "browser";
    protected String browsePass = "browser";
 
@@ -140,6 +146,9 @@ public class MQTT5TestSupport extends ActiveMQTestBase {
    protected String fullUser = "user";
    protected String fullPass = "pass";
 
+   protected String noDeleteUser = "noDelete";
+   protected String noDeletePass = "noDelete";
+
    @Rule
    public TestName name = new TestName();
 
@@ -202,12 +211,16 @@ public class MQTT5TestSupport extends ActiveMQTestBase {
          // User additions
          securityManager.getConfiguration().addUser(noprivUser, noprivPass);
          securityManager.getConfiguration().addRole(noprivUser, "nothing");
+         securityManager.getConfiguration().addUser(createAddressUser, createAddressPass);
+         securityManager.getConfiguration().addRole(createAddressUser, "createAddress");
          securityManager.getConfiguration().addUser(browseUser, browsePass);
          securityManager.getConfiguration().addRole(browseUser, "browser");
          securityManager.getConfiguration().addUser(guestUser, guestPass);
          securityManager.getConfiguration().addRole(guestUser, "guest");
          securityManager.getConfiguration().addUser(fullUser, fullPass);
          securityManager.getConfiguration().addRole(fullUser, "full");
+         securityManager.getConfiguration().addUser(noDeleteUser, noDeleteUser);
+         securityManager.getConfiguration().addRole(noDeleteUser, "noDelete");
 
          // Configure roles
          HierarchicalRepository<Set<Role>> securityRepository = server.getSecurityRepository();
@@ -216,6 +229,8 @@ public class MQTT5TestSupport extends ActiveMQTestBase {
          value.add(new Role("browser", false, false, false, false, false, false, false, true, false, false));
          value.add(new Role("guest", false, true, false, false, false, false, false, true, false, false));
          value.add(new Role("full", true, true, true, true, true, true, true, true, true, true));
+         value.add(new Role("createAddress", false, false, false, false, false, false, false, false, true, false));
+         value.add(new Role("noDelete", true, true, true, false, true, false, true, true, true, true));
          securityRepository.addMatch("#", value);
 
          server.getConfiguration().setSecurityEnabled(true);
@@ -339,7 +354,12 @@ public class MQTT5TestSupport extends ActiveMQTestBase {
 
    protected Queue getSubscriptionQueue(String TOPIC) {
       try {
-         return ((LocalQueueBinding)server.getPostOffice().getBindingsForAddress(SimpleString.toSimpleString(TOPIC)).getBindings().toArray()[0]).getQueue();
+         Object[] array = server.getPostOffice().getBindingsForAddress(SimpleString.toSimpleString(TOPIC)).getBindings().toArray();
+         if (array.length == 0) {
+            return null;
+         } else {
+            return ((LocalQueueBinding)array[0]).getQueue();
+         }
       } catch (Exception e) {
          e.printStackTrace();
          return null;
@@ -347,9 +367,17 @@ public class MQTT5TestSupport extends ActiveMQTestBase {
    }
 
    protected Queue getSubscriptionQueue(String TOPIC, String clientId) {
+      return getSubscriptionQueue(TOPIC, clientId, null);
+   }
+
+   protected Queue getSubscriptionQueue(String TOPIC, String clientId, String sharedSubscriptionName) {
       try {
          for (Binding b : server.getPostOffice().getMatchingBindings(SimpleString.toSimpleString(TOPIC))) {
-            if (((LocalQueueBinding)b).getQueue().getName().startsWith(SimpleString.toSimpleString(clientId))) {
+            if (sharedSubscriptionName != null) {
+               if (((LocalQueueBinding)b).getQueue().getName().startsWith(SimpleString.toSimpleString(sharedSubscriptionName))) {
+                  return ((LocalQueueBinding)b).getQueue();
+               }
+            } else if (((LocalQueueBinding)b).getQueue().getName().startsWith(SimpleString.toSimpleString(clientId))) {
                return ((LocalQueueBinding)b).getQueue();
             }
          }
